/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef JAC_AP_VEDSERVICE_VEDSERVICE_SKELETON_H
#define JAC_AP_VEDSERVICE_VEDSERVICE_SKELETON_H

#include "ara/com/internal/skeleton/skeleton_adapter.h"
#include "ara/com/internal/skeleton/event_adapter.h"
#include "ara/com/internal/skeleton/field_adapter.h"
#include "ara/com/internal/skeleton/method_adapter.h"
#include "ara/com/crc_verification.h"
#include "jac/ap/vedservice/vedservice_common.h"
#include <cstdint>

namespace jac {
namespace ap {
namespace vedservice {
namespace skeleton {
namespace events
{
    using VED4DPCEvent = ara::com::internal::skeleton::event::EventAdapter<::dpc::VED4DPC>;
    using VED2FusionEgoMotionEvent = ara::com::internal::skeleton::event::EventAdapter<::dpc::VED4FusionEgoMotion>;
    static constexpr ara::com::internal::EntityId VEDServiceVED4DPCEventId = 55898U; //VED4DPCEvent_event_hash
    static constexpr ara::com::internal::EntityId VEDServiceVED2FusionEgoMotionEventId = 57422U; //VED2FusionEgoMotionEvent_event_hash
}

namespace methods
{
}

namespace fields
{
}

class VEDServiceSkeleton {
private:
    std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> skeletonAdapter;
    void ConstructSkeleton(const ara::com::MethodCallProcessingMode mode)
    {
        if (mode == ara::com::MethodCallProcessingMode::kEvent) {
            if (!(skeletonAdapter->SetMethodThreadNumber(skeletonAdapter->GetMethodThreadNumber(0U), 1024U))) {
#ifndef NOT_SUPPORT_EXCEPTIONS
                ara::core::ErrorCode errorcode(ara::com::ComErrc::kNetworkBindingFailure);
                throw ara::com::ComException(std::move(errorcode));
#else
                std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
            }
        }
        const ara::core::Result<void> resultVED4DPCEvent = VED4DPCEvent.Initialize();
        ThrowError(resultVED4DPCEvent);
        const ara::core::Result<void> resultVED2FusionEgoMotionEvent = VED2FusionEgoMotionEvent.Initialize();
        ThrowError(resultVED2FusionEgoMotionEvent);
    }

    VEDServiceSkeleton& operator=(const VEDServiceSkeleton&) = delete;

    static void ThrowError(const ara::core::Result<void>& result)
    {
        if (!(result.HasValue())) {
#ifndef NOT_SUPPORT_EXCEPTIONS
            ara::core::ErrorCode errorcode(result.Error());
            throw ara::com::ComException(std::move(errorcode));
#else
            std::cerr << "Error: Not support exception, create skeleton failed!" << std::endl;
#endif
        }
    }
public:
    class ConstructionToken {
    public:
        explicit ConstructionToken(const ara::com::InstanceIdentifier& instanceId,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::vedservice::VEDService::ServiceIdentifier, instanceId, mode)),
              VED4DPCEvent(ptr->GetSkeleton(), events::VEDServiceVED4DPCEventId, ::jac::ap::vedservice::VEDService::ServiceIdentifier, instanceId),
              VED2FusionEgoMotionEvent(ptr->GetSkeleton(), events::VEDServiceVED2FusionEgoMotionEventId, ::jac::ap::vedservice::VEDService::ServiceIdentifier, instanceId){
        }

        explicit ConstructionToken(const ara::core::InstanceSpecifier& instanceSpec,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::vedservice::VEDService::ServiceIdentifier, instanceSpec, mode)),
              VED4DPCEvent(ptr->GetSkeleton(), events::VEDServiceVED4DPCEventId, ::jac::ap::vedservice::VEDService::ServiceIdentifier, instanceSpec),
              VED2FusionEgoMotionEvent(ptr->GetSkeleton(), events::VEDServiceVED2FusionEgoMotionEventId, ::jac::ap::vedservice::VEDService::ServiceIdentifier, instanceSpec){
        }

        explicit ConstructionToken(const ara::com::InstanceIdentifierContainer instanceContainer,
            const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
            : processMode(mode),
              ptr(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::vedservice::VEDService::ServiceIdentifier, instanceContainer, mode)),
              VED4DPCEvent(ptr->GetSkeleton(), events::VEDServiceVED4DPCEventId, ::jac::ap::vedservice::VEDService::ServiceIdentifier, instanceContainer),
              VED2FusionEgoMotionEvent(ptr->GetSkeleton(), events::VEDServiceVED2FusionEgoMotionEventId, ::jac::ap::vedservice::VEDService::ServiceIdentifier, instanceContainer){
        }

        ConstructionToken(ConstructionToken&& other)
            : processMode(std::move(other.processMode)),
              ptr(std::move(other.ptr)),
              VED4DPCEvent(std::move(other.VED4DPCEvent)),
              VED2FusionEgoMotionEvent(std::move(other.VED2FusionEgoMotionEvent)){
        }
        ConstructionToken& operator=(ConstructionToken && other)
        {
            if (&other != this) {
                processMode = std::move(other.processMode);
                ptr = std::move(other.ptr);
                VED4DPCEvent = std::move(other.VED4DPCEvent);
                VED2FusionEgoMotionEvent = std::move(other.VED2FusionEgoMotionEvent);
            }
            return *this;
        }
        ConstructionToken(const ConstructionToken&) = delete;
        ConstructionToken& operator = (const ConstructionToken&) = delete;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> GetSkeletonAdapter()
        {
            return std::move(ptr);
        }
        events::VED4DPCEvent GetVED4DPCEvent()
        {
            return std::move(VED4DPCEvent);
        }
        events::VED2FusionEgoMotionEvent GetVED2FusionEgoMotionEvent()
        {
            return std::move(VED2FusionEgoMotionEvent);
        }
        ara::core::Result<void> Initialize()
        {
            ara::core::Result<void> initResult;
            do {
                if (processMode == ara::com::MethodCallProcessingMode::kEvent) {
                    if(!ptr->SetMethodThreadNumber(ptr->GetMethodThreadNumber(0U), 1024U)) {
                        ara::core::ErrorCode errorCode(ara::com::ComErrc::kNetworkBindingFailure);
                        initResult.EmplaceError(errorCode);
                        break;
                    }
                }
                initResult = VED4DPCEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
                initResult = VED2FusionEgoMotionEvent.Initialize();
                if (!initResult.HasValue()) {
                    break;
                }
            } while(false);

            return initResult;
        }
    private:
        ara::com::MethodCallProcessingMode processMode;
        std::unique_ptr<ara::com::internal::skeleton::SkeletonAdapter> ptr;
        events::VED4DPCEvent VED4DPCEvent;
        events::VED2FusionEgoMotionEvent VED2FusionEgoMotionEvent;
    };
    explicit VEDServiceSkeleton(const ara::com::InstanceIdentifier& instanceId,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        : skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::vedservice::VEDService::ServiceIdentifier, instanceId, mode)),
          VED4DPCEvent(skeletonAdapter->GetSkeleton(), events::VEDServiceVED4DPCEventId, ::jac::ap::vedservice::VEDService::ServiceIdentifier, instanceId),
          VED2FusionEgoMotionEvent(skeletonAdapter->GetSkeleton(), events::VEDServiceVED2FusionEgoMotionEventId, ::jac::ap::vedservice::VEDService::ServiceIdentifier, instanceId){
        ConstructSkeleton(mode);
    }

    explicit VEDServiceSkeleton(const ara::core::InstanceSpecifier& instanceSpec,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::vedservice::VEDService::ServiceIdentifier, instanceSpec, mode)),
          VED4DPCEvent(skeletonAdapter->GetSkeleton(), events::VEDServiceVED4DPCEventId, ::jac::ap::vedservice::VEDService::ServiceIdentifier, instanceSpec),
          VED2FusionEgoMotionEvent(skeletonAdapter->GetSkeleton(), events::VEDServiceVED2FusionEgoMotionEventId, ::jac::ap::vedservice::VEDService::ServiceIdentifier, instanceSpec){
        ConstructSkeleton(mode);
    }

    explicit VEDServiceSkeleton(const ara::com::InstanceIdentifierContainer instanceContainer,
                           const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
        :skeletonAdapter(std::make_unique<ara::com::internal::skeleton::SkeletonAdapter>(::jac::ap::vedservice::VEDService::ServiceIdentifier, instanceContainer, mode)),
          VED4DPCEvent(skeletonAdapter->GetSkeleton(), events::VEDServiceVED4DPCEventId, ::jac::ap::vedservice::VEDService::ServiceIdentifier, instanceContainer),
          VED2FusionEgoMotionEvent(skeletonAdapter->GetSkeleton(), events::VEDServiceVED2FusionEgoMotionEventId, ::jac::ap::vedservice::VEDService::ServiceIdentifier, instanceContainer){
        ConstructSkeleton(mode);
    }

    VEDServiceSkeleton(const VEDServiceSkeleton&) = delete;

    VEDServiceSkeleton(VEDServiceSkeleton&&) = default;
    VEDServiceSkeleton& operator=(VEDServiceSkeleton&&) = default;
    VEDServiceSkeleton(ConstructionToken&& token) noexcept
        : skeletonAdapter(token.GetSkeletonAdapter()),
          VED4DPCEvent(token.GetVED4DPCEvent()),
          VED2FusionEgoMotionEvent(token.GetVED2FusionEgoMotionEvent()){
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifier instanceId,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceId, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::core::InstanceSpecifier instanceSpec,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceSpec, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        ara::com::InstanceIdentifierContainer instanceIdContainer,
        const ara::com::MethodCallProcessingMode mode = ara::com::MethodCallProcessingMode::kEvent)
    {
        ConstructionToken token(instanceIdContainer, mode);
        const auto initResult = token.Initialize();
        ara::core::Result<ConstructionToken> result(std::move(token));
        if (!initResult.HasValue()) {
            const ara::core::ErrorCode errorCode(initResult.Error());
            result.EmplaceError(errorCode);
        }
        return result;
    }

    virtual ~VEDServiceSkeleton()
    {
        StopOfferService();
    }

    void OfferService()
    {
        skeletonAdapter->RegisterE2EErrorHandler(&VEDServiceSkeleton::E2EErrorHandler, *this);
        skeletonAdapter->OfferService();
    }
    void StopOfferService()
    {
        skeletonAdapter->StopOfferService();
    }
    ara::core::Future<bool> ProcessNextMethodCall()
    {
        return skeletonAdapter->ProcessNextMethodCall();
    }
    bool SetMethodThreadNumber(const std::uint16_t& number, const std::uint16_t& queueSize)
    {
        return skeletonAdapter->SetMethodThreadNumber(number, queueSize);
    }

    virtual void E2EErrorHandler(ara::com::e2e::E2EErrorCode, ara::com::e2e::DataID, ara::com::e2e::MessageCounter){}

    events::VED4DPCEvent VED4DPCEvent;
    events::VED2FusionEgoMotionEvent VED2FusionEgoMotionEvent;
};
} // namespace skeleton
} // namespace vedservice
} // namespace ap
} // namespace jac

#endif // JAC_AP_VEDSERVICE_VEDSERVICE_SKELETON_H
